package com.example.fuzzControll.tools;

import com.example.fuzzControll.conf.AflnetProperties;
import com.example.fuzzControll.conf.SpringContextUtil;
import com.example.fuzzControll.exception.FileException;
import com.example.fuzzControll.pojo.vo.AflnetResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;

@Slf4j
public class FileTools {
    AflnetProperties properties = (AflnetProperties) SpringContextUtil.getBean("AflnetProperties");

    public void load(MultipartFile file) throws FileException {
        if (file == null) {
            throw new FileException("upload file is null !");
        }
        try (InputStream inputStream = file.getInputStream();
             FileOutputStream outputStream = new FileOutputStream(properties.getSeedPath() + "/" + file.getOriginalFilename());) {
            BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
            BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(outputStream);
            byte[] buffer = new byte[1024];
            int bytesRead;
            while ((bytesRead = bufferedInputStream.read(buffer)) != -1) {
                bufferedOutputStream.write(buffer, 0, bytesRead);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new FileException("write file error !");
        }

    }

    /**
     * 文件备份至数据库
     *
     * @param filename
     * @return
     */
    public byte[] fileReadAndBackup(String filename) {
        File file = new File(properties.getOutputPath() + "/" + filename);
        try (
                FileInputStream inputStream = new FileInputStream(file);
                BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);) {
            if (file.length() > Integer.MAX_VALUE) {
                throw new IOException();
            }
            byte[] buffer = new byte[(int) file.length()];
            int bytesRead;
            while ((bytesRead = bufferedInputStream.read(buffer)) != -1) {
                bufferedInputStream.read(buffer, 0, bytesRead);
            }
            return buffer;
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException("File is to large " + file.getName());
        }
    }

    /**
     * 读取数据库文件至本地
     */
    public void loadFileInLocal(AflnetResult aflnetResult, String fileLoadPath) {
        File file = new File(fileLoadPath+"/"+aflnetResult.getFileName());
        try (FileOutputStream fileOutputStream = new FileOutputStream(file);
             BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(fileOutputStream);
             ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(aflnetResult.getFile());
             BufferedInputStream bufferedInputStream = new BufferedInputStream(byteArrayInputStream);) {
            byte[] buffer = new byte[1024];
            int bytesRead;
            while ((bytesRead = bufferedInputStream.read(buffer)) != -1) {
                bufferedOutputStream.write(buffer, 0, bytesRead);
            }
        } catch (FileNotFoundException e) {
            throw new FileException("Cannot find path:" + fileLoadPath+"! Maybe there is no permission for this file!");
        } catch (IOException e) {
            throw new FileException("Load file failed!");
        }
    }
}